@@ -107,14 +107,11 @@ add_vevent (ECalBackendExchange *cbex,
 			icalcomponent_set_dtend (comp, itt);
 		}
 
-		if (!strcmp (x_name, "X-MICROSOFT-CDO-BUSYSTATUS")) {
+		if (!strcmp (x_name, "X-MICROSOFT-CDO-BUSYSTATUS") && !transp) {
 			/* It seems OWA sometimes doesn't set the
 			 * TRANSP property, so set it from the busy
 			 * status.
 			 */
-			if (transp)
-				icalcomponent_remove_property (comp,transp);
-
 			if (!strcmp (x_val, "BUSY"))
 				transp = icalproperty_new_transp (ICAL_TRANSP_OPAQUE);
 			else if (!strcmp (x_val, "FREE"))
@@ -160,7 +157,7 @@ add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 {
 	const char *start, *end;
 	char *ical_body;
-	icalcomponent *comp, *subcomp;
+	icalcomponent *comp, *subcomp, *new_comp;
 	icalcomponent_kind kind;
 	gboolean status;
 
@@ -194,7 +191,11 @@ add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 	subcomp = icalcomponent_get_first_component (
 		comp, ICAL_VEVENT_COMPONENT);
 	while (subcomp) {
-		add_vevent (cbex, href, lastmod, icalcomponent_new_clone (subcomp));
+		new_comp = icalcomponent_new_clone (subcomp);
+		if (new_comp) {
+			add_vevent (cbex, href, lastmod, new_comp);
+			icalcomponent_free (new_comp);
+		}
 		subcomp = icalcomponent_get_next_component (
 			comp, ICAL_VEVENT_COMPONENT);
 	}
@@ -315,6 +316,7 @@ get_changed_events (ECalBackendExchange *cbex, const char *since)
 		add_ical (cbex, result->href, modtime,
 			  ical_data->data, ical_data->len);
 	}
+	// g_byte_array_free (ical_data);
 	status = e2k_result_iter_free (iter);
 
 	if (!SOUP_STATUS_IS_SUCCESSFUL (status)) {
@@ -364,7 +366,11 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal,
 		backend, cal, only_if_exists, username, password);
 	if (status != GNOME_Evolution_Calendar_Success)
 		return status;
-
+#ifdef OFFLINE_SUPPORTED
+	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (backend))) {
+		return GNOME_Evolution_Calendar_Success;
+	}
+#endif
 	/* Now load the rest of the calendar items */
 	status = get_changed_events (E_CAL_BACKEND_EXCHANGE (backend), NULL);
 	if (status == E2K_HTTP_OK)
@@ -408,6 +414,7 @@ static ECalBackendSyncStatus
 create_object (ECalBackendSync *backend, EDataCal *cal,
 	       char **calobj, char **uid)
 {
+	/* FIXME : Return some value in uid */
 	ECalBackendExchangeCalendar *cbexc;
 	icalcomponent *icalcomp, *real_icalcomp;
 	icalcomponent_kind kind;
@@ -428,7 +435,12 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE_CALENDAR (cbexc), GNOME_Evolution_Calendar_InvalidObject);
 	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_InvalidObject);
-	
+#ifdef OFFLINE_SUPPORTED	
+	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (backend))) {
+		/* FIXME */
+		return GNOME_Evolution_Calendar_InvalidObject;
+	}
+#endif
 	/* check for permission denied:: priv->writable??
 	   ....
 	 */
@@ -529,6 +541,7 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	e_cal_component_commit_sequence (comp);
 	*calobj = e_cal_component_get_as_string (comp);
 	if (!*calobj) {
+		icalcomponent_free (icalcomp);
 		g_object_unref (comp);
 		icalcomponent_free (cbdata->vcal_comp);
 		g_free (cbdata);
@@ -539,10 +552,9 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	icalcomponent_foreach_tzid (real_icalcomp, add_timezone_cb, cbdata);
 	icalcomponent_add_component (cbdata->vcal_comp, real_icalcomp);	
 	
-	
 	body = icalcomponent_as_ical_string (cbdata->vcal_comp);
 	body_crlf = e_cal_backend_exchange_lf_to_crlf (body);	
-	
+		
 	summary = icalcomponent_get_summary (real_icalcomp);
 	if (!summary)
 		summary = "";
@@ -570,8 +582,20 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 						NULL, NULL, "message/rfc822", 
 						msg, strlen (msg), &location, &ru_header);	
 
-	if (http_status != E2K_HTTP_CREATED)
+	g_free (date);
+	g_free (from);
+	g_free (body_crlf);
+	g_free (msg); 
+	icalcomponent_free (cbdata->vcal_comp);	// not sure
+	g_free (cbdata);
+
+	if (http_status != E2K_HTTP_CREATED) {
+		icalcomponent_free (icalcomp);
+		g_object_unref (comp);
+		g_free (location);
+		g_free (lastmod);
 		return GNOME_Evolution_Calendar_OtherError;
+	}
 
 	/*add object*/
 	e_cal_backend_exchange_add_object (E_CAL_BACKEND_EXCHANGE (cbexc), location, lastmod, icalcomp);
@@ -580,8 +604,10 @@ create_object (ECalBackendSync *backend, EDataCal *cal,
 	e_cal_backend_ref_categories (E_CAL_BACKEND (cbexc), categories);
 	e_cal_component_free_categories_list (categories);
 	
+	icalcomponent_free (icalcomp);
+	g_object_unref (comp);
 	g_free (lastmod);
-	/*cleanup ?*/	
+	g_free (location);
 	
 	return GNOME_Evolution_Calendar_Success;
 }
@@ -692,10 +718,11 @@ update_x_properties (ECalBackendExchange *cbex, ECalComponent *comp)
 static ECalBackendSyncStatus
 modify_object (ECalBackendSync *backend, EDataCal *cal,
 	       const char *calobj, CalObjModType mod,
-	       char **old_object)
+	       char **old_object, char **new_object)
 {
 	return modify_object_with_href (backend, cal, calobj, mod, old_object, NULL);
 }
+
 static ECalBackendSyncStatus
 modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	       const char *calobj, CalObjModType mod,
@@ -703,10 +730,10 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 {
 	ECalBackendExchangeCalendar *cbexc;
 	ECalBackendExchangeComponent *ecomp;
-	icalcomponent *icalcomp, *real_icalcomp;
-	ECalComponent *comp, *tmp_comp, *old_comp;
+	icalcomponent *icalcomp, *real_icalcomp, *updated_icalcomp;
+	ECalComponent *real_ecomp, *cached_ecomp, *updated_ecomp;
 	const char *comp_uid;
-	char *comp_str;
+	char *updated_ecomp_str, *real_comp_str;
 	char *body, *body_crlf, *msg;
 	struct icaltimetype last_modified;
 	icalcomponent_kind kind;
@@ -731,7 +758,6 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	kind = icalcomponent_isa (icalcomp);
 	
 	if (kind != e_cal_backend_get_kind (E_CAL_BACKEND (backend))) {
-		
 		icalcomponent_free (icalcomp);
 		return GNOME_Evolution_Calendar_InvalidObject;
 	}
@@ -744,26 +770,41 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 		return GNOME_Evolution_Calendar_ObjectNotFound;
 	}
 	
-	comp = e_cal_component_new ();
-	e_cal_component_set_icalcomponent (comp, icalcomp);
+	cached_ecomp = e_cal_component_new ();
+	e_cal_component_set_icalcomponent (cached_ecomp, ecomp->icomp);
+	*old_object = e_cal_component_get_as_string (cached_ecomp);
+	
+	updated_ecomp = e_cal_component_new ();
+	e_cal_component_set_icalcomponent (updated_ecomp, icalcomp);
 
-	update_x_properties (E_CAL_BACKEND_EXCHANGE (cbexc), comp);
+	update_x_properties (E_CAL_BACKEND_EXCHANGE (cbexc), updated_ecomp);
 	
 	last_modified = icaltime_from_timet (time (NULL), 0);
-	e_cal_component_set_last_modified (comp, &last_modified);
+	e_cal_component_set_last_modified (updated_ecomp, &last_modified);
 	
-	comp_str = e_cal_component_get_as_string (comp);
-	icalcomp = icalparser_parse_string (comp_str);
-	g_free (comp_str);
-	if (!icalcomp) {
-		g_object_unref (comp);
+	updated_ecomp_str = e_cal_component_get_as_string (updated_ecomp);
+	updated_icalcomp = icalparser_parse_string (updated_ecomp_str);
+	g_free (updated_ecomp_str);
+	if (!updated_icalcomp) {
+		g_object_unref (updated_ecomp);
 		return GNOME_Evolution_Calendar_OtherError;
 	}
 
-	tmp_comp = e_cal_component_new ();
-	if (!e_cal_component_set_icalcomponent (tmp_comp, icalcomp)) {
-		g_object_unref (comp);
-		g_object_unref (tmp_comp);
+	/* Remove X parameters from properties */
+	/* This is specifically for X-EVOLUTION-END-DATE, 
+	   but removing anything else is probably ok too */
+	for (icalprop = icalcomponent_get_first_property (updated_icalcomp, ICAL_ANY_PROPERTY);
+	     icalprop != NULL;
+	     icalprop = icalcomponent_get_next_property (updated_icalcomp, ICAL_ANY_PROPERTY))
+	{
+		icalproperty_remove_parameter (icalprop, ICAL_X_PARAMETER);
+	}
+
+	real_ecomp = e_cal_component_new ();
+	if (!e_cal_component_set_icalcomponent (real_ecomp, updated_icalcomp)) {
+		g_object_unref (real_ecomp);
+		g_object_unref (updated_ecomp);
+		icalcomponent_free (updated_icalcomp);
 		return GNOME_Evolution_Calendar_OtherError;
 	}
 
@@ -772,7 +813,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	cbdata->vcal_comp = e_cal_util_new_top_level ();
 	cbdata->cal = cal;	
 
-	e_cal_component_get_dtstart (tmp_comp, &dt);
+	e_cal_component_get_dtstart (real_ecomp, &dt);
 	if (dt.value->is_date) {
 		icaltimezone *zone;
 
@@ -785,32 +826,30 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 		dt.value->hour = dt.value->minute = dt.value->second = 0;
 		dt.value->zone = zone;
 		dt.tzid = icaltimezone_get_tzid (zone);
-		e_cal_component_set_dtstart (tmp_comp, &dt);
-		e_cal_component_free_datetime (&dt);
+		e_cal_component_set_dtstart (real_ecomp, &dt);
 
-		e_cal_component_get_dtend (tmp_comp, &dt);
+		e_cal_component_get_dtend (real_ecomp, &dt);
 		dt.value->is_date = FALSE;
 		dt.value->is_utc = FALSE;
 		dt.value->hour = dt.value->minute = dt.value->second = 0;
 		dt.value->zone = zone;
 		dt.tzid = icaltimezone_get_tzid (zone);
-		e_cal_component_set_dtend (tmp_comp, &dt);
+		e_cal_component_set_dtend (real_ecomp, &dt);
 	}
-	e_cal_component_free_datetime (&dt);
 
 	/* Fix UNTIL date in a simple recurrence */
-	if (e_cal_component_has_recurrences (tmp_comp)
-	    && e_cal_component_has_simple_recurrence (tmp_comp)) {
+	if (e_cal_component_has_recurrences (real_ecomp)
+	    && e_cal_component_has_simple_recurrence (real_ecomp)) {
 		GSList *rrule_list;
 		struct icalrecurrencetype *r;
 		
-		e_cal_component_get_rrule_list (tmp_comp, &rrule_list);
+		e_cal_component_get_rrule_list (real_ecomp, &rrule_list);
 		r = rrule_list->data;
 
 		if (!icaltime_is_null_time (r->until) && r->until.is_date) {
 			icaltimezone *from_zone, *to_zone;
 			
-			e_cal_component_get_dtstart (tmp_comp, &dt);
+			e_cal_component_get_dtstart (real_ecomp, &dt);
 
 			if (dt.tzid == NULL)
 				from_zone = icaltimezone_get_utc_timezone ();
@@ -830,38 +869,27 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 			icaltimezone_convert_time (&r->until, from_zone, to_zone);
 			r->until.is_utc = TRUE;
 			
-			e_cal_component_set_rrule_list (tmp_comp, rrule_list);			
-
-			e_cal_component_free_datetime (&dt);
+			e_cal_component_set_rrule_list (real_ecomp, rrule_list);			
 		}
 
 		e_cal_component_free_recur_list (rrule_list);
 	}
 
 
-	/* Remove X parameters from properties */
-	/* This is specifically for X-EVOLUTION-END-DATE, 
-	   but removing anything else is probably ok too */
-	for (icalprop = icalcomponent_get_first_property (icalcomp, ICAL_ANY_PROPERTY);
-	     icalprop != NULL;
-	     icalprop = icalcomponent_get_next_property (icalcomp, ICAL_ANY_PROPERTY))
-	{
-		icalproperty_remove_parameter (icalprop, ICAL_X_PARAMETER);
-	}
 	
 	/* add the timezones information and the component itself
 	   to the VCALENDAR object */
-	e_cal_component_commit_sequence (tmp_comp);
-	comp_str = e_cal_component_get_as_string (tmp_comp);
-	if (!comp_str) {
-		g_object_unref (tmp_comp);
-		g_object_unref (comp);
+	e_cal_component_commit_sequence (real_ecomp);
+	real_comp_str = e_cal_component_get_as_string (real_ecomp);
+	if (!real_comp_str) {
+		g_object_unref (real_ecomp);
+		g_object_unref (updated_ecomp);	
 		icalcomponent_free (cbdata->vcal_comp);
 		g_free (cbdata);
 		return GNOME_Evolution_Calendar_OtherError;
 	}
-	real_icalcomp = icalparser_parse_string (comp_str);
-	g_free (comp_str);
+	real_icalcomp = icalparser_parse_string (real_comp_str);
+	g_free (real_comp_str);
 
 	icalcomponent_foreach_tzid (real_icalcomp, add_timezone_cb, cbdata);
 	icalcomponent_add_component (cbdata->vcal_comp, real_icalcomp);
@@ -870,7 +898,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	body_crlf = e_cal_backend_exchange_lf_to_crlf (body);
 	
 	date = e_cal_backend_exchange_make_timestamp_rfc822 (time (NULL));
-	from = e_cal_backend_exchange_get_from_string (backend, comp);	
+	from = e_cal_backend_exchange_get_from_string (backend, updated_ecomp);	
 	
 	summary = icalcomponent_get_summary (real_icalcomp);
 	if (!summary)
@@ -895,17 +923,13 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 	g_free (from);
 	g_free (body_crlf);
 
-	old_comp = e_cal_component_new ();
-	e_cal_component_set_icalcomponent (old_comp, icalcomponent_new_clone (ecomp->comp));
-	*old_object = e_cal_component_get_as_string (old_comp);
-	
 	/*unref the old set of categories*/
-	e_cal_component_get_categories_list (old_comp, &categories);
+	e_cal_component_get_categories_list (cached_ecomp, &categories);
 	e_cal_backend_unref_categories (E_CAL_BACKEND (cbexc), categories);
 	e_cal_component_free_categories_list (categories);
 	
 	/*ref the new set of categories*/
-	e_cal_component_get_categories_list (tmp_comp, &categories);
+	e_cal_component_get_categories_list (real_ecomp, &categories);
 	e_cal_backend_ref_categories (E_CAL_BACKEND (cbexc), categories);
 	e_cal_component_free_categories_list (categories);
 
@@ -925,9 +949,9 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 							real_icalcomp, mod);
 	
 	g_free (msg);
-	g_object_unref (comp);
-	g_object_unref (tmp_comp);
-	g_object_unref (old_comp);
+	g_object_unref (real_ecomp);
+	g_object_unref (updated_ecomp);
+	g_object_unref (cached_ecomp);
 	icalcomponent_free (cbdata->vcal_comp);
 	g_free (cbdata);
 
@@ -937,7 +961,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 static ECalBackendSyncStatus
 remove_object (ECalBackendSync *backend, EDataCal *cal,
 	       const char *uid, const char *rid, CalObjModType mod,
-	       char **object)
+	       char **old_object, char **object)
 {
 	ECalBackendExchangeCalendar *cbexc;
 	ECalBackendExchangeComponent *ecomp;
@@ -959,7 +983,7 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 		return GNOME_Evolution_Calendar_ObjectNotFound;
 		
 	comp = e_cal_component_new ();
-	e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (ecomp->comp));
+	e_cal_component_set_icalcomponent (comp, ecomp->icomp);
 	*object = e_cal_component_get_as_string (comp);
 	
 	switch (mod) {
@@ -973,18 +997,20 @@ remove_object (ECalBackendSync *backend, EDataCal *cal,
 				e_cal_backend_unref_categories (E_CAL_BACKEND (cbexc), categories);
 				e_cal_component_free_categories_list (categories);
 				
-				if (e_cal_backend_exchange_remove_object (E_CAL_BACKEND_EXCHANGE (cbexc), uid))
+				if (e_cal_backend_exchange_remove_object (E_CAL_BACKEND_EXCHANGE (cbexc), uid)) {
+						g_object_unref (comp);
 						return GNOME_Evolution_Calendar_Success;
+				}
 			}
 			break;
 		case CALOBJ_MOD_THIS:
 			/*remove_instance and modify */
 			if (rid && *rid) {
 				time_rid = icaltime_from_string (rid);
-				e_cal_util_remove_instances (ecomp->comp, time_rid, mod);
+				e_cal_util_remove_instances (ecomp->icomp, time_rid, mod);
 			}
-			calobj  = (char *) icalcomponent_as_ical_string (ecomp->comp);
-			ebs_status = modify_object (backend, cal, calobj, mod, &obj);
+			calobj  = (char *) icalcomponent_as_ical_string (ecomp->icomp);
+			ebs_status = modify_object (backend, cal, calobj, mod, &obj, NULL);
 			if (ebs_status != GNOME_Evolution_Calendar_Success)
 				goto error;
 			
@@ -1043,9 +1069,9 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 		case ICAL_METHOD_REPLY:
 			if (get_exchange_comp (E_CAL_BACKEND_EXCHANGE (cbexc), uid)) {
 				char *old_object;
-
+				
 				calobj = (char *) icalcomponent_as_ical_string (subcomp);
-				status = modify_object (backend, cal, calobj, CALOBJ_MOD_THIS, &old_object);
+				status = modify_object (backend, cal, calobj, CALOBJ_MOD_THIS, &old_object, NULL);
 				if (status != GNOME_Evolution_Calendar_Success)
 					goto error;
 
@@ -1069,8 +1095,8 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 
 		case ICAL_METHOD_CANCEL:
 			calobj = (char *) icalcomponent_as_ical_string (subcomp);
-			status = remove_object (backend, cal, uid, rid, CALOBJ_MOD_THIS, &calobj);
-			e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), uid, calobj);
+			status = remove_object (backend, cal, uid, rid, CALOBJ_MOD_THIS, &calobj, NULL);
+			e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), uid, calobj, NULL);
 			break;
 		default:
 			status = GNOME_Evolution_Calendar_UnsupportedMethod;
@@ -1078,9 +1104,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 		}
 		g_object_unref (comp);
 	}
-
 	g_list_free (comps);
-	return status;
 		
  error:	
 	if (comp)
@@ -1138,17 +1162,18 @@ book_resource (ECalBackendExchange *cbex,
 	gcstatus = e2k_global_catalog_lookup (
 		gc, NULL, E2K_GLOBAL_CATALOG_LOOKUP_BY_EMAIL, resource_email,
 		E2K_GLOBAL_CATALOG_LOOKUP_MAILBOX, &entry);
+
 	switch (gcstatus) {
-	case E2K_GLOBAL_CATALOG_OK:
-		break;
+		case E2K_GLOBAL_CATALOG_OK:
+			break;
 												 
-	case E2K_GLOBAL_CATALOG_NO_SUCH_USER:
-		retval = E_CAL_BACKEND_EXCHANGE_BOOKING_NO_SUCH_USER;
-		goto cleanup;
+		case E2K_GLOBAL_CATALOG_NO_SUCH_USER:
+			retval = E_CAL_BACKEND_EXCHANGE_BOOKING_NO_SUCH_USER;
+			goto cleanup;
 												 
-	default:
-		retval = E_CAL_BACKEND_EXCHANGE_BOOKING_ERROR;
-		goto cleanup;
+		default:
+			retval = E_CAL_BACKEND_EXCHANGE_BOOKING_ERROR;
+			goto cleanup;
 	}
 												 
 	top_uri = exchange_account_get_foreign_uri (cbex->account,
@@ -1218,13 +1243,13 @@ book_resource (ECalBackendExchange *cbex,
 			new_text.value = g_strdup_printf ("Cancelled");
 		new_text.altrep = NULL;
 		e_cal_component_set_summary (E_CAL_COMPONENT (comp), &new_text);
-												 
+
 		e_cal_component_set_transparency (E_CAL_COMPONENT (comp), E_CAL_COMPONENT_TRANSP_TRANSPARENT);
 		calobj = (char *) e_cal_component_get_as_string (comp);
 		rid = (char *) e_cal_component_get_recurid_as_string (comp);
-		status = remove_object (E_CAL_BACKEND_SYNC (cbex), cal, uid, rid, CALOBJ_MOD_THIS, &calobj);
-		e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbex), uid, calobj);
-		g_free (calobj);
+		status = remove_object (E_CAL_BACKEND_SYNC (cbex), cal, uid, rid, CALOBJ_MOD_THIS, &calobj, NULL);
+		e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbex), uid, calobj, NULL);
+		g_free (calobj); 
 	} else {
 		/* Check that the new appointment doesn't conflict with any
 		 * existing appointment.
@@ -1262,28 +1287,26 @@ book_resource (ECalBackendExchange *cbex,
 						     &prop_name, 1, rn, NULL, FALSE);
 		g_free (startz);
 		g_free (endz);
+
 		result = e2k_result_iter_next (iter);
 		if (result) {
 			cal_uid = e2k_properties_get_prop (result[0].props, E2K_PR_CALENDAR_UID);
 		}
 		if (result && cal_uid) {
 			retval = E_CAL_BACKEND_EXCHANGE_BOOKING_BUSY;
+
+			status = e2k_result_iter_free (iter);
+
+			if (!E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
+				if (status == E2K_HTTP_UNAUTHORIZED) 
+					retval = E_CAL_BACKEND_EXCHANGE_BOOKING_PERMISSION_DENIED;
+				else 
+					retval = E_CAL_BACKEND_EXCHANGE_BOOKING_ERROR;
+			}
+			e2k_restriction_unref (rn);
 			goto cleanup;
 		}
-		status = e2k_result_iter_free (iter);
-
-		g_free (startz);
-		g_free (endz);
 		e2k_restriction_unref (rn);
-		if (!E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
-			if (status == E2K_HTTP_UNAUTHORIZED) {
-				retval = E_CAL_BACKEND_EXCHANGE_BOOKING_PERMISSION_DENIED;
-				goto cleanup;
-			} else {
-				retval = E_CAL_BACKEND_EXCHANGE_BOOKING_ERROR;
-				goto cleanup;
-			}
-		}
 	}
 												 
 	/* We're good. Book it. */
@@ -1298,8 +1321,8 @@ book_resource (ECalBackendExchange *cbex,
 		if (status == GNOME_Evolution_Calendar_Success) {
 			e_cal_backend_notify_object_modified (E_CAL_BACKEND (cbex), old_object, calobj);
 			retval = E_CAL_BACKEND_EXCHANGE_BOOKING_OK;
-			g_free (old_object);
 		}
+		g_free (old_object);
 	} else {
 		status = create_object (E_CAL_BACKEND_SYNC (cbex), cal, &calobj, &returned_uid);
 		if (status == GNOME_Evolution_Calendar_Success) {
@@ -1316,6 +1339,7 @@ book_resource (ECalBackendExchange *cbex,
 		g_free (cal_uri);
 	if (top_uri)
 		g_free (top_uri);
+	g_free (calobj);
 					
 	return retval;
 }
@@ -1335,6 +1359,12 @@ send_objects (ECalBackendSync *backend, EDataCal *cal,
 												 
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE (cbex), 
 				GNOME_Evolution_Calendar_InvalidObject);
+
+#ifdef OFFLINE_SUPPORTED
+	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (cbex))) { 
+		return GNOME_Evolution_Calendar_InvalidObject;
+	}
+#endif
 												 
 	*users = NULL;
 	*modified_calobj = NULL;
@@ -1523,7 +1553,7 @@ discard_alarm (ECalBackendSync *backend, EDataCal *cal,
 		return GNOME_Evolution_Calendar_ObjectNotFound;
 
 	ecomp = e_cal_component_new ();
-	e_cal_component_set_icalcomponent (ecomp, icalcomponent_new_clone (ecbexcomp->comp));
+	e_cal_component_set_icalcomponent (ecomp, ecbexcomp->icomp);
 	if (!e_cal_component_has_recurrences (ecomp))
 	{
 		e_cal_component_remove_alarm (ecomp, auid);
